@@ -1,5 +1,5 @@
 /* tc-iq2000.c -- Assembler for the Sitera IQ2000.
-   Copyright (C) 2003-2024 Free Software Foundation, Inc.
+   Copyright (C) 2003-2025 Free Software Foundation, Inc.
 
    This file is part of GAS, the GNU Assembler.
 
@@ -105,6 +105,8 @@ struct iq2000_hi_fixup
 /* The list of unmatched HI relocs.  */
 static struct iq2000_hi_fixup * iq2000_hi_fixup_list;
 
+/* Macro hash table, which we will add to.  */
+extern struct htab *macro_hash;
 
 const char md_shortopts[] = "";
 const struct option md_longopts[] =
@@ -277,7 +279,9 @@ iq2000_add_macro (const char *  name,
 	}
     }
 
-  (void) add_macro (macro, true);
+  str_hash_insert (macro_hash, macro->name, macro, 1);
+
+  macro_defined = 1;
 }
 
 static void
@@ -678,7 +682,7 @@ s_iq2000_set (int x ATTRIBUTE_UNUSED)
   char *name = input_line_pointer, ch;
   char *save_ILP = input_line_pointer;
 
-  while (!is_end_of_line[(unsigned char) *input_line_pointer])
+  while (!is_end_of_stmt (*input_line_pointer))
     input_line_pointer++;
   ch = *input_line_pointer;
   *input_line_pointer = '\0';
@@ -799,7 +803,7 @@ s_iq2000_end (int x ATTRIBUTE_UNUSED)
   symbolS *p;
   int maybe_text;
 
-  if (!is_end_of_line[(unsigned char) *input_line_pointer])
+  if (!is_end_of_stmt (*input_line_pointer))
     {
       p = get_symbol ();
       demand_empty_rest_of_line ();
