@@ -1,6 +1,6 @@
 /* tc-loongarch.c -- Assemble for the LoongArch ISA
 
-   Copyright (C) 2021-2024 Free Software Foundation, Inc.
+   Copyright (C) 2021-2025 Free Software Foundation, Inc.
    Contributed by Loongson Ltd.
 
    This file is part of GAS.
@@ -320,115 +320,98 @@ loongarch_after_parse_args ()
 
   /* Init ilp32/lp64 registers names.  */
   if (!r_htab)
-    r_htab = str_htab_create (), str_hash_insert (r_htab, "", 0, 0);
+    r_htab = str_htab_create ();
   if (!r_deprecated_htab)
-    r_deprecated_htab = str_htab_create (),
-			str_hash_insert (r_deprecated_htab, "", 0, 0);
+    r_deprecated_htab = str_htab_create ();
   /* Init cfi registers alias.  */
   if (!cfi_r_htab)
-    cfi_r_htab = str_htab_create (), str_hash_insert (cfi_r_htab, "", 0, 0);
+    cfi_r_htab = str_htab_create ();
 
   r_abi_names = loongarch_r_normal_name;
   for (i = 0; i < ARRAY_SIZE (loongarch_r_normal_name); i++)
     {
-      str_hash_insert (r_htab, loongarch_r_normal_name[i],
-		       (void *) (i + 1), 0);
-      str_hash_insert (cfi_r_htab, loongarch_r_normal_name[i],
-		       (void *) (i + 1), 0);
+      str_hash_insert_int (r_htab, loongarch_r_normal_name[i], i, 0);
+      str_hash_insert_int (cfi_r_htab, loongarch_r_normal_name[i], i, 0);
     }
   /* Init ilp32/lp64 registers alias.  */
   r_abi_names = loongarch_r_alias;
   for (i = 0; i < ARRAY_SIZE (loongarch_r_alias); i++)
     {
-      str_hash_insert (r_htab, loongarch_r_alias[i],
-		       (void *) (i + 1), 0);
-      str_hash_insert (cfi_r_htab, loongarch_r_alias[i],
-		       (void *) (i + 1), 0);
+      str_hash_insert_int (r_htab, loongarch_r_alias[i], i, 0);
+      str_hash_insert_int (cfi_r_htab, loongarch_r_alias[i], i, 0);
     }
 
   for (i = 0; i < ARRAY_SIZE (loongarch_r_alias_1); i++)
-    str_hash_insert (r_htab, loongarch_r_alias_1[i], (void *) (i + 1), 0);
+    str_hash_insert_int (r_htab, loongarch_r_alias_1[i], i, 0);
 
   for (i = 0; i < ARRAY_SIZE (loongarch_r_alias_deprecated); i++)
-    str_hash_insert (r_deprecated_htab, loongarch_r_alias_deprecated[i],
-	(void *) (i + 1), 0);
+    str_hash_insert_int (r_deprecated_htab, loongarch_r_alias_deprecated[i],
+			 i, 0);
 
   /* The .cfi directive supports register aliases without the "$" prefix.  */
   for (i = 0; i < ARRAY_SIZE (loongarch_r_cfi_name); i++)
     {
-      str_hash_insert (cfi_r_htab, loongarch_r_cfi_name[i],
-		       (void *)(i + 1), 0);
-      str_hash_insert (cfi_r_htab, loongarch_r_cfi_name_alias[i],
-		       (void *)(i + 1), 0);
+      str_hash_insert_int (cfi_r_htab, loongarch_r_cfi_name[i], i, 0);
+      str_hash_insert_int (cfi_r_htab, loongarch_r_cfi_name_alias[i], i, 0);
     }
 
   if (!cr_htab)
-    cr_htab = str_htab_create (), str_hash_insert (cr_htab, "", 0, 0);
+    cr_htab = str_htab_create ();
 
   for (i = 0; i < ARRAY_SIZE (loongarch_cr_normal_name); i++)
-    str_hash_insert (cr_htab, loongarch_cr_normal_name[i], (void *) (i + 1), 0);
+    str_hash_insert_int (cr_htab, loongarch_cr_normal_name[i], i, 0);
 
   /* Init single/double float registers names.  */
   if (LARCH_opts.ase_sf || LARCH_opts.ase_df)
     {
       if (!f_htab)
-	f_htab = str_htab_create (), str_hash_insert (f_htab, "", 0, 0);
+	f_htab = str_htab_create ();
       if (!f_deprecated_htab)
-	f_deprecated_htab = str_htab_create (),
-			    str_hash_insert (f_deprecated_htab, "", 0, 0);
+	f_deprecated_htab = str_htab_create ();
       if (!cfi_f_htab)
-	cfi_f_htab = str_htab_create (), str_hash_insert (cfi_f_htab, "", 0, 0);
+	cfi_f_htab = str_htab_create ();
 
       f_abi_names = loongarch_f_normal_name;
       for (i = 0; i < ARRAY_SIZE (loongarch_f_normal_name); i++)
 	{
-	  str_hash_insert (f_htab, loongarch_f_normal_name[i],
-			   (void *) (i + 1), 0);
-	  str_hash_insert (cfi_f_htab, loongarch_f_normal_name[i],
-			   (void *) (i + 1), 0);
+	  str_hash_insert_int (f_htab, loongarch_f_normal_name[i], i, 0);
+	  str_hash_insert_int (cfi_f_htab, loongarch_f_normal_name[i], i, 0);
 	}
       /* Init float-ilp32/lp64 registers alias.  */
       f_abi_names = loongarch_f_alias;
       for (i = 0; i < ARRAY_SIZE (loongarch_f_alias); i++)
 	{
-	  str_hash_insert (f_htab, loongarch_f_alias[i],
-			   (void *) (i + 1), 0);
-	  str_hash_insert (cfi_f_htab, loongarch_f_alias[i],
-			   (void *) (i + 1), 0);
+	  str_hash_insert_int (f_htab, loongarch_f_alias[i], i, 0);
+	  str_hash_insert_int (cfi_f_htab, loongarch_f_alias[i], i, 0);
 	}
       for (i = 0; i < ARRAY_SIZE (loongarch_f_alias_deprecated); i++)
-	str_hash_insert (f_deprecated_htab, loongarch_f_alias_deprecated[i],
-	    (void *) (i + 1), 0);
+	str_hash_insert_int (f_deprecated_htab, loongarch_f_alias_deprecated[i],
+			     i, 0);
 
   /* The .cfi directive supports register aliases without the "$" prefix.  */
   for (i = 0; i < ARRAY_SIZE (loongarch_f_cfi_name); i++)
     {
-      str_hash_insert (cfi_f_htab, loongarch_f_cfi_name[i],
-		       (void *)(i + 1), 0);
-      str_hash_insert (cfi_f_htab, loongarch_f_cfi_name_alias[i],
-		       (void *)(i + 1), 0);
+      str_hash_insert_int (cfi_f_htab, loongarch_f_cfi_name[i], i, 0);
+      str_hash_insert_int (cfi_f_htab, loongarch_f_cfi_name_alias[i], i, 0);
     }
 
       if (!fc_htab)
-	fc_htab = str_htab_create (), str_hash_insert (fc_htab, "", 0, 0);
+	fc_htab = str_htab_create ();
 
       for (i = 0; i < ARRAY_SIZE (loongarch_fc_normal_name); i++)
-	str_hash_insert (fc_htab, loongarch_fc_normal_name[i], (void *) (i + 1),
-			 0);
+	str_hash_insert_int (fc_htab, loongarch_fc_normal_name[i], i, 0);
 
       if (!fcn_htab)
-	fcn_htab = str_htab_create (), str_hash_insert (fcn_htab, "", 0, 0);
+	fcn_htab = str_htab_create ();
 
       for (i = 0; i < ARRAY_SIZE (loongarch_fc_numeric_name); i++)
-	str_hash_insert (fcn_htab, loongarch_fc_numeric_name[i], (void *) (i + 1),
-			 0);
+	str_hash_insert_int (fcn_htab, loongarch_fc_numeric_name[i], i, 0);
 
       if (!c_htab)
-	c_htab = str_htab_create (), str_hash_insert (c_htab, "", 0, 0);
+	c_htab = str_htab_create ();
 
       for (i = 0; i < ARRAY_SIZE (loongarch_c_normal_name); i++)
-	str_hash_insert (c_htab, loongarch_c_normal_name[i], (void *) (i + 1),
-			 0);
+	str_hash_insert_int (c_htab, loongarch_c_normal_name[i], i, 0);
 
     }
 
@@ -436,20 +419,18 @@ loongarch_after_parse_args ()
   if (LARCH_opts.ase_lsx)
     {
       if (!v_htab)
-	v_htab = str_htab_create (), str_hash_insert (v_htab, "", 0, 0);
+	v_htab = str_htab_create ();
       for (i = 0; i < ARRAY_SIZE (loongarch_v_normal_name); i++)
-	str_hash_insert (v_htab, loongarch_v_normal_name[i], (void *) (i + 1),
-			 0);
+	str_hash_insert_int (v_htab, loongarch_v_normal_name[i], i, 0);
     }
 
   /* Init lasx registers names.  */
   if (LARCH_opts.ase_lasx)
     {
       if (!x_htab)
-	x_htab = str_htab_create (), str_hash_insert (x_htab, "", 0, 0);
+	x_htab = str_htab_create ();
       for (i = 0; i < ARRAY_SIZE (loongarch_x_normal_name); i++)
-	str_hash_insert (x_htab, loongarch_x_normal_name[i], (void *) (i + 1),
-			 0);
+	str_hash_insert_int (x_htab, loongarch_x_normal_name[i], i, 0);
     }
 
 }
@@ -600,7 +581,7 @@ static void
 s_loongarch_option (int x ATTRIBUTE_UNUSED)
 {
   char *name = input_line_pointer, ch;
-  while (!is_end_of_line[(unsigned char) *input_line_pointer])
+  while (!is_end_of_stmt (*input_line_pointer))
     ++input_line_pointer;
   ch = *input_line_pointer;
   *input_line_pointer = '\0';
@@ -934,15 +915,15 @@ loongarch_args_parser_can_match_arg_helper (char esc_ch1, char esc_ch2,
 	}
       break;
     case 'r':
-      imm = (intptr_t) str_hash_find (r_htab, arg);
-      ip->match_now = 0 < imm;
-      ret = imm - 1;
+      imm = str_hash_find_int (r_htab, arg);
+      ip->match_now = 0 <= imm;
+      ret = imm;
       if (ip->match_now)
 	break;
       /* Handle potential usage of deprecated register aliases.  */
-      imm = (intptr_t) str_hash_find (r_deprecated_htab, arg);
-      ip->match_now = 0 < imm;
-      ret = imm - 1;
+      imm = str_hash_find_int (r_deprecated_htab, arg);
+      ip->match_now = 0 <= imm;
+      ret = imm;
       /* !ip->expand_from_macro: avoiding duplicate output warnings,
 	 only the first macro output warning.  */
       if (ip->match_now && !ip->expand_from_macro)
@@ -953,23 +934,21 @@ loongarch_args_parser_can_match_arg_helper (char esc_ch1, char esc_ch2,
       switch (esc_ch2)
 	{
 	case 'c':
-	  imm = (intptr_t) str_hash_find (fc_htab, arg);
-	  if (0 >= imm)
-	    {
-	      imm = (intptr_t) str_hash_find (fcn_htab, arg);
-	    }
+	  imm = str_hash_find_int (fc_htab, arg);
+	  if (0 > imm)
+	    imm = str_hash_find_int (fcn_htab, arg);
 	  break;
 	default:
-	  imm = (intptr_t) str_hash_find (f_htab, arg);
+	  imm = str_hash_find_int (f_htab, arg);
 	}
-      ip->match_now = 0 < imm;
-      ret = imm - 1;
+      ip->match_now = 0 <= imm;
+      ret = imm;
       if (ip->match_now && !ip->expand_from_macro)
 	break;
       /* Handle potential usage of deprecated register aliases.  */
-      imm = (intptr_t) str_hash_find (f_deprecated_htab, arg);
-      ip->match_now = 0 < imm;
-      ret = imm - 1;
+      imm = str_hash_find_int (f_deprecated_htab, arg);
+      ip->match_now = 0 <= imm;
+      ret = imm;
       if (ip->match_now)
 	as_warn (_("register alias %s is deprecated, use %s instead"),
 		 arg, f_abi_names[ret]);
@@ -978,23 +957,23 @@ loongarch_args_parser_can_match_arg_helper (char esc_ch1, char esc_ch2,
       switch (esc_ch2)
 	{
 	case 'r':
-	  imm = (intptr_t) str_hash_find (cr_htab, arg);
+	  imm = str_hash_find_int (cr_htab, arg);
 	  break;
 	default:
-	  imm = (intptr_t) str_hash_find (c_htab, arg);
+	  imm = str_hash_find_int (c_htab, arg);
 	}
-      ip->match_now = 0 < imm;
-      ret = imm - 1;
+      ip->match_now = 0 <= imm;
+      ret = imm;
       break;
     case 'v':
-      imm = (intptr_t) str_hash_find (v_htab, arg);
-      ip->match_now = 0 < imm;
-      ret = imm - 1;
+      imm = str_hash_find_int (v_htab, arg);
+      ip->match_now = 0 <= imm;
+      ret = imm;
       break;
     case 'x':
-      imm = (intptr_t) str_hash_find (x_htab, arg);
-      ip->match_now = 0 < imm;
-      ret = imm - 1;
+      imm = str_hash_find_int (x_htab, arg);
+      ip->match_now = 0 <= imm;
+      ret = imm;
       break;
     case '\0':
       ip->all_match = ip->match_now;
@@ -1077,8 +1056,7 @@ get_loongarch_opcode (struct loongarch_cl_insn *insn)
 	      if ((!it->include || (it->include && *it->include))
 		  && (!it->exclude || (it->exclude && !(*it->exclude)))
 		  && !(it->pinfo & INSN_DIS_ALIAS))
-		str_hash_insert (ase->name_hash_entry, it->name,
-				 (void *) it, 0);
+		str_hash_insert (ase->name_hash_entry, it->name, it, 0);
 	    }
 	}
 
@@ -1420,13 +1398,14 @@ loongarch_assemble_INSNs (char *str, unsigned int expand_from_macro)
       if (*str == '\0')
 	break;
 
-      struct loongarch_cl_insn the_one = { 0 };
+      struct loongarch_cl_insn the_one;
+      memset (&the_one, 0, sizeof (the_one));
       the_one.name = str;
       the_one.expand_from_macro = expand_from_macro;
 
-      for (; *str && *str != ' '; str++)
+      for (; *str && !is_whitespace (*str); str++)
 	;
-      if (*str == ' ')
+      if (is_whitespace (*str))
 	*str++ = '\0';
 
       loongarch_split_args_by_comma (str, the_one.arg_strs);
@@ -1521,6 +1500,29 @@ loongarch_force_relocation (struct fix *fixp)
   return generic_force_reloc (fixp);
 }
 
+/* If subsy of BFD_RELOC32/64 and PC in same segment, and without relax
+   or PC at start of subsy or with relax but sub_symbol_segment not in
+   SEC_CODE, we generate 32/64_PCREL.  */
+bool
+loongarch_force_relocation_sub_local (fixS *fixp, segT sec ATTRIBUTE_UNUSED)
+{
+  return !(LARCH_opts.thin_add_sub
+	   && (fixp->fx_r_type == BFD_RELOC_32
+	       || fixp->fx_r_type == BFD_RELOC_64)
+	   && (!LARCH_opts.relax
+	       || (S_GET_VALUE (fixp->fx_subsy)
+		   == fixp->fx_frag->fr_address + fixp->fx_where)
+	       || (S_GET_SEGMENT (fixp->fx_subsy)->flags & SEC_CODE) == 0));
+}
+
+/* Postpone text-section label subtraction calculation until linking, since
+   linker relaxations might change the deltas.  */
+bool
+loongarch_force_relocation_sub_same(fixS *fixp ATTRIBUTE_UNUSED, segT sec)
+{
+  return LARCH_opts.relax && (sec->flags & SEC_CODE) != 0;
+}
+
 static void fix_reloc_insn (fixS *fixP, bfd_vma reloc_val, char *buf)
 {
   reloc_howto_type *howto;
@@ -1879,9 +1881,10 @@ md_estimate_size_before_relax (fragS *fragp, asection *sec)
 arelent *
 tc_gen_reloc (asection *section ATTRIBUTE_UNUSED, fixS *fixp)
 {
-  arelent *reloc = (arelent *) xmalloc (sizeof (arelent));
+  arelent *reloc;
 
-  reloc->sym_ptr_ptr = (asymbol **) xmalloc (sizeof (asymbol *));
+  reloc = notes_alloc (sizeof (arelent));
+  reloc->sym_ptr_ptr = notes_alloc (sizeof (asymbol *));
   *reloc->sym_ptr_ptr = symbol_get_bfdsym (fixp->fx_addsy);
   reloc->address = fixp->fx_frag->fr_address + fixp->fx_where;
   reloc->addend = fixp->fx_offset;
@@ -1912,12 +1915,12 @@ tc_loongarch_regname_to_dw2regnum (char *regname)
   int reg;
 
   /* Look up in the general purpose register table.  */
-  if ((reg = (intptr_t) str_hash_find (cfi_r_htab, regname)) > 0)
-    return reg - 1;
+  if ((reg = str_hash_find_int (cfi_r_htab, regname)) >= 0)
+    return reg;
 
   /* Look up in the floating point register table.  */
-  if ((reg = (intptr_t) str_hash_find (cfi_f_htab, regname)) > 0)
-    return reg + 31;
+  if ((reg = str_hash_find_int (cfi_f_htab, regname)) >= 0)
+    return reg + 32;
 
   as_bad (_("unknown register `%s`"), regname);
   return -1;
