@@ -1,5 +1,5 @@
 /* tc-tilegx.c -- Assemble for a Tile-Gx chip.
-   Copyright (C) 2011-2024 Free Software Foundation, Inc.
+   Copyright (C) 2011-2025 Free Software Foundation, Inc.
 
    This file is part of GAS, the GNU Assembler.
 
@@ -275,7 +275,7 @@ md_begin (void)
   /* Initialize special operator hash table.  */
   special_operator_hash = str_htab_create ();
 #define INSERT_SPECIAL_OP(name)					\
-  str_hash_insert (special_operator_hash, #name, (void *) O_##name, 0)
+  str_hash_insert_int (special_operator_hash, #name, O_##name, 0)
 
   INSERT_SPECIAL_OP (hw0);
   INSERT_SPECIAL_OP (hw1);
@@ -285,7 +285,7 @@ md_begin (void)
   INSERT_SPECIAL_OP (hw1_last);
   INSERT_SPECIAL_OP (hw2_last);
   /* hw3_last is a convenience alias for the equivalent hw3.  */
-  str_hash_insert (special_operator_hash, "hw3_last", (void *) O_hw3, 0);
+  str_hash_insert_int (special_operator_hash, "hw3_last", O_hw3, 0);
   INSERT_SPECIAL_OP (hw0_got);
   INSERT_SPECIAL_OP (hw0_last_got);
   INSERT_SPECIAL_OP (hw1_last_got);
@@ -329,14 +329,14 @@ md_begin (void)
     {
       char buf[64];
 
-      str_hash_insert (main_reg_hash, tilegx_register_names[i],
-		       (void *) (long) (i | CANONICAL_REG_NAME_FLAG), 0);
+      str_hash_insert_int (main_reg_hash, tilegx_register_names[i],
+			   i | CANONICAL_REG_NAME_FLAG, 0);
 
       /* See if we should insert a noncanonical alias, like r63.  */
       sprintf (buf, "r%d", i);
       if (strcmp (buf, tilegx_register_names[i]) != 0)
-	str_hash_insert (main_reg_hash, xstrdup (buf),
-			 (void *) (long) (i | NONCANONICAL_REG_NAME_FLAG), 0);
+	str_hash_insert_int (main_reg_hash, xstrdup (buf),
+			     i | NONCANONICAL_REG_NAME_FLAG, 0);
     }
 }
 
@@ -1027,10 +1027,10 @@ tilegx_parse_name (char *name, expressionS *e, char *nextcharP)
   else
     {
       /* Look up the operator in our table.  */
-      void* val = str_hash_find (special_operator_hash, name);
-      if (val == 0)
+      int opint = str_hash_find_int (special_operator_hash, name);
+      if (opint < 0)
 	return 0;
-      op = (operatorT)(long)val;
+      op = opint;
     }
 
   /* Restore old '(' and skip it.  */
@@ -1083,23 +1083,17 @@ tilegx_parse_name (char *name, expressionS *e, char *nextcharP)
 static void
 parse_reg_expression (expressionS* expression)
 {
-  char *regname;
-  char terminating_char;
-  void *pval;
-  int regno_and_flags;
-  int regno;
-
   /* Zero everything to make sure we don't miss any flags.  */
   memset (expression, 0, sizeof *expression);
 
-  terminating_char = get_symbol_name (&regname);
+  char *regname;
+  char terminating_char = get_symbol_name (&regname);
 
-  pval = str_hash_find (main_reg_hash, regname);
-  if (pval == NULL)
+  int regno_and_flags = str_hash_find_int (main_reg_hash, regname);
+  if (regno_and_flags < 0)
     as_bad (_("Expected register, got '%s'."), regname);
 
-  regno_and_flags = (int)(size_t)pval;
-  regno = EXTRACT_REGNO(regno_and_flags);
+  int regno = EXTRACT_REGNO(regno_and_flags);
 
   if ((regno_and_flags & NONCANONICAL_REG_NAME_FLAG)
       && require_canonical_reg_names)
@@ -1159,9 +1153,9 @@ parse_operands (const char *opcode_name,
 
       if (i + 1 < num_operands)
 	{
-	  int separator = (unsigned char)*input_line_pointer++;
+	  char separator = *input_line_pointer++;
 
-	  if (is_end_of_line[separator] || (separator == '}'))
+	  if (is_end_of_stmt (separator) || (separator == '}'))
 	    {
 	      as_bad (_("Too few operands to '%s'."), opcode_name);
 	      return;
@@ -1169,7 +1163,7 @@ parse_operands (const char *opcode_name,
 	  else if (separator != ',')
 	    {
 	      as_bad (_("Unexpected character '%c' after operand %d to %s."),
-		      (char)separator, i + 1, opcode_name);
+		      separator, i + 1, opcode_name);
 	      return;
 	    }
 	}
@@ -1196,7 +1190,7 @@ parse_operands (const char *opcode_name,
 	}
     }
 
-  if (!is_end_of_line[(unsigned char)*input_line_pointer])
+  if (!is_end_of_stmt (*input_line_pointer))
     {
       switch (*input_line_pointer)
 	{
@@ -1729,8 +1723,8 @@ tc_gen_reloc (asection *sec ATTRIBUTE_UNUSED, fixS *fixp)
 {
   arelent *reloc;
 
-  reloc = XNEW (arelent);
-  reloc->sym_ptr_ptr = XNEW (asymbol *);
+  reloc = notes_alloc (sizeof (arelent));
+  reloc->sym_ptr_ptr = notes_alloc (sizeof (asymbol *));
   *reloc->sym_ptr_ptr = symbol_get_bfdsym (fixp->fx_addsy);
   reloc->address = fixp->fx_frag->fr_address + fixp->fx_where;
 
